#
# UIM delete device
# - create list of devices with cs_key
# - read device(s) to be deleted 
# - match device with cs_key (and optionally origin & dedicated)
# - if MCS/SSR poller device do not continue
# - call back discovery_server remove_master_device_by_cskey
# - delete all qos data for this device (optionally matched by origin)
#   (if dedicated is used no delete qos performed)
# - delete all alarms in nas_transaction_log and nas_transaction_summary
# - delete SSR/MCS records and QoS
# - create robotlist with the corresponding hub
# - use callback to the correct hub: removerobot
#
# Version 3.3 02/05/2019 - Luc Christiaens 
#
# *** Pre-Req: ***
# - you need:  nimsoft_generic.pm and nimsoft_generic.dat
# -   when you run in perl mode, copy nimsoft_generic.pm in your /perl/lib directory (else it's compiled into the exe) or in the directory with the .pl source
# -   copy nimsoft_generic.dat in the same directory as the perl module (.pl or .exe)
# - Edit nimsoft_generic.dat and change all variables, they will be common between several UIM/Nimsoft Perl utilities
#
# Updates:
# - 19/01/16: changed from specific server names to regex server string
# - 20/01/16: add -y 1: mssql 2: mysql (default:1) 
#           : sql_type in nimsoft_generic
# - 02/02/16: add option -q: delete QoS (y/n) default y 
# - 05/02/16: if user is "trusted" use Windows authentication to logon to MSSQL
# - 10/02/16: add -o: regex filter on origin
# - 11/02/16: add -c: regex filter on dedicated (if dedicated is used QoS delete is set to n)
# - 26/02/16: change QoS query to include sources that are expressed in ip address to match name given in -h (adaption thanks to sql query from jonhcw)
# - 04/05/16: Nestor Falcon: added switch "-r" to enable/disable regex expressions on switches -h,-c,-o (default: n)
# - 09/05/16: Nestor Falcon: include deletion of data from tables BN_QoS and DN_QoS.  Include deletion of SSR data
# - 14/04/16: no simulation mode (-s"n") can also be indicated by -k"y" (kill)
# - 18/05/16: add -a, delete alarms in nas_transaction_log and nas_transaction_summary (y,n) default: y
# - 19/05/16: add -i, delete device in infrastructure manager (y,n) default: y
# - 20/05/16: add -v, verbose after sql delete, this will show the number of rows deleted (y,n) default: y
# - 22/06/16: add b and d QoS tables (merge Nestor updates)
# - 24/06/16: add -m, delete MCS/SSR records (y,n) default:y (pre 8.4 this doesn't exist) (merge Nestor updates)
#           : add -x: stop processing when no match is found in cm_computer_system, default: y
# - 11/08/16: stay in the domain defined in nimsoft_generic.dat 
# - 01/09/16: changed to #!/opt/nimbus/bin/perl
#           : tested on centos7 accessing windows uim server & accessing Mariadb on another centos
# - 15/12/16: info msg about delete BN table had no value (wrong variable)
# - 02/01/17: option -i (delete device in infrastructure manager) did obtain robot/hub list via call-backs, but we do it now with 1 sql query, this will only works for device in the table: CM_NIMBUS_ROBOT and CM_DEVICE
# - 02/02/17: add use_https in nimsoft_generic.dat/pm
# - 28/02/17: 8.5.1 does not have a MCS model device anymore, put these sections in comment 
# - 10/03/17: MCS option -m is now default: n
# - 13/03/17: add -n: nas callback: nameservice_delete to delete device also from nas address table
# - 13/04/17: add variable $optioni that makes it possible to select the old callback method (y) or the new sql method (n). Default: n
# - 28/04/17: add with(nolock) for all tables in all sql queries
# - 16/05/17: change in display only the reference from cm_computer_table to cm_computer_system
# - 25/08/17: change delete_mcs back to y
# - 18/09/17: add option -d"p", this will only print in output the PU command that you can use to simulate the otherwise internally executed callback: remove_master_device_by_cskey (also in simulate mode)
#             (so you can check if it's the command or the perl program that creates a possible problem)  
#             in debug mode (-d"y") don't generate teh device evaluation anymore, this generates too many garbage
# - 26/09/17: add option -f where you can define the path and file that contains the servers to delete, 1 server per line. Use / in path
# - 28/09/17: add sleep of 5 seconds before step 7 delete device from nas address table to avoid communication error when deleting multilple devices
#           : set delete nas default to n (like this delete is never documented, not really sure if this has an advantage?)   
# - 09/10/17: -d"p" will also generate the pu for removerobot (if a matching hub is found)
# - 20/10/17: moved sql query to create list robot with matching hub for step6 to begin of script
# - 27/10/17: step6 removerobot command was not constructed ok when accessing a remote hub
# - 22/03/18: if a device has multiple cs_id's we will try to remove them all to obtain a clean result (that's why in the display all server names are displayed in lowercase, so all case differences for the same server are ignored)
# - 01/05/18: add extra message if sql login failure. This message can be used by probe_decom (that uses this perl to do the decommission of a device)
# - 26/03/18: add option -w"probe" to work with probe_decom
# - 04/11/18: test 9.0.2 with delete 7 fake servers receive msg: error_code: PDS_I 2 0 => this means that the return code is: 0
# - 14/12/18: add option -p (= prevent rediscovery), options y or n, default: n
#             (this will add in 9.0.2 an entry in table: CM_BLACKLIST_DEVICE and CM_BLACKLIST_COMPUTER_SYSTEM)
#             (in pre-9.0.2 this comes in a file: excluded_devices.csv from directory: discovery_server)
# - 14/04/19: added $sth->finish; in loop while deleting alarms, while deleting a lot of devices in 1 run (+3000) you could end in a deadlock
# - 02/05/19: add more explicit print statements to do the debugging (each message has his own unique number)
#           : add new/updated MCS code from Nestor Falcon to delete MCS profiles 
# - 04/05/19: add option -t: verbose_sql so that you can use verbose without receiving all the details of the deletes qos entries (y,n) default: n 
#           : rebuilded Perl source library from scratch to update documentation how to build teh Windows Perl environment 
# - 06/05/19: in simulate mode (-s"y") and verbode mode (-v"y") and IM delete (-i"y") show the detail hub lookup logic
# - 07/05/19: the removerobot callback needed the robotname in the exact case
#           : added the usage of 3 new perl packages to be able to use color statements (see doc appendix for detailed instructions how to setup a valid/good perl environment)
# - 16/05/19: no color when running as/from a probe
#
# ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
# - This program is only an example program that can be used on your own risk
# ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
#
# To Do
#
#


#!/opt/nimbus/bin/perl
# some packages we use
use DBI;
# use DBD::ODBC;
use Getopt::Std;
# crypt
use Crypt::RC4;
use MIME::Base64;
# needed for callback
use Nimbus::API;
use Nimbus::Session;
use Nimbus::CFG;
use Nimbus::PDS;
$crkey="ab_def%hij&5";
$pgm="Nimsoft_Delete_Device (v3.3)";
# --- to use colors in the print statements
use Win32::Console::ANSI;
use Term::ANSIColor;
# --- change directory to directory where perl source is located to find our nimsoft_generic.dat file
#use Cwd 'chdir';
use Cwd;
use File::Basename;
chdir dirname $0;
$curdir=cwd();
# --- UIM generic package (copy in teh same directory as your Perl source)
use nimsoft_generic;

%pds_type = (
	"PDS_I", 1,
	"PDS_PI", 2,
	"PDS_PPI", 3,
	"PDS_RGI", 4,
	"PDS_RGPI", 5,
	"PDS_CH", 6,
	"PDS_PCH", 7,
	"PDS_PPCH", 8,
	"PDS_RGCH", 9,
	"PDS_RGPCH", 10,
	"PDS_DUM_L", 11,
	"PDS_DUM_PL", 12,
	"PDS_DUM_PPL", 13,
	"PDS_DUM_RGL", 14,
	"PDS_DUM_RGPL", 15,
	"PDS_F", 16,
	"PDS_PF", 17,
	"PDS_PPF", 18,
	"PDS_RGF", 19,
	"PDS_RGPF", 20,
	"PDS_PDS", 21,
	"PDS_VOID", 22,
	"PDS_SEP", 23,
	"PDS_PPDS", 24,
	"PDS_CPCH", 25,
	"PDS_CPDS", 26,
	"PDS_INT64", 27
);

%pds_type2string = reverse %pds_type;

# --- needed vor uim calls
$request_timeout = 30;
$request_retries = 1;
$timeout = 10;

# --- option -i via old callback method? (=y) (default new sql method (=n))
$optioni="n";

# --- get parameters
&getparams;

# --- variables
# - table for our found computers in cm_computer_system
@tab1 = ();
# - table for our found computers and related hub name
@tab2 = ();
# - table for our found hubs and related address
@tab3 = ();
# - table for finding the robotname in the exact case
@tab4 = ();
# - counter for read computers
$count=0;
# - variable to create cs_id list
$cslist="";
# - counter for number of cs_ids added
$cscount=0;
# - counter for the number of servers requested to delete
$reqcount=0;
# - counter for hosts deleted
$countm=0;
# - count for sql lines
$sqlcount=0;
# - counter for QoS
$count9=1;

#
# --- step1 - read -h with server(s) 
#

# print "step 1.01: Start processing devices\n";

# --- do we have an input file?
if ($file_input ne '')
  {
    &read_input;
    $file_list=$display_server;
  }
$servlist=$file_list;
$servlist=lc($servlist);

print "step 1.03: We added device string: $servlist\n";
if ($filter_origin ne '') 
 {
  print "step 1.04: filter on origin: $filter_origin\n";
 }
if ($filter_dedicated ne '') 
 {
  print "step 1.05: filter on dedicated: $filter_dedicated (delete QoS is set to: n)\n";
 }
if ($useregex eq 'y') 
 {
  print "step 1.06: Defined filters will be used in REGEX format (-r\"y\")\n";
 }


#
# --- connect DB ---
#

if ($sqltype eq 'my')
  {
    $dbh = DBI->connect("DBI:mysql:database=$sql_db;host=$sql_server", "$sql_user", "$sql_password") ;
  }
if ($sqltype eq 'ms')
  {
    if ($trusted eq 'yes')
      {
        $dbh = DBI->connect("DBI:ODBC:Driver={SQL Server};Server=$sql_server;Database=$sql_db;Trusted_Connection=Yes;") ;
#        if ($verbose eq 'y') {print "* mssql trusted connection to connect to: $sql_server\n";}
      }
    else
      {
        $dbh = DBI->connect("DBI:ODBC:Driver={SQL Server};Server=$sql_server;Database=$sql_db;UID=$sql_user;PWD=$sql_password");
#        if ($verbose eq 'y') {print "* mssql connection to connect to server: $sql_server db: $sql_db with userid: $sql_user\n";}
      }
  }
if (!$dbh)
    {
        if ($probe eq 'y')
         {
            print "step 1.07: Could not connect to server: $sql_server db: $sql_db error: $DBI::errstr\n";
         }
        else
         { 
            print (color("black on_white "),"step 1.08: Could not connect to server: $sql_server db: $sql_db error: $DBI::errstr",color("reset"),"\n");
         }
       exit 12;
    }

# $dbh = DBI->connect("DBI:ODBC:Driver={SQL Server};Server=$sql_server;Database=$sql_db;UID=$sql_user;PWD=$sql_password") ;

#
# -- Get the cs_id of Poller devices and Model devices (MCS). If any of the devices to be deleted is a Poller or Model Device, a msg is display and command aborts.
#

# --- bypass MCS model server (does not exist anymore in 8.5.1, so we removed it)

step2:
#
# --- step2 - get all devices from CM_COMPUTER_SYSTEM and find a match with our regex server list to create a cs_id list in return)
# (This match will not use regex unless the switch -r is set to yes)
#

# --- prepare sql query (nimbus_type: 1=robot 2=hub) => no test on type so that we have all entries for a device
# or: select name, cs_key, nimbus_type,origin,dedicated from CM_COMPUTER_SYSTEM order by name
# dedicated: Device, VirtualMachine, Host, HostSystem

# (mssql and mysql same)
if ($sqltype eq 'ms')
  {
     $sql = "select cs_id,name, cs_key, nimbus_type, origin, dedicated from CM_COMPUTER_SYSTEM with(nolock) order by name";
  }
else
  {
     $sql = "select cs_id,name, cs_key, nimbus_type, origin, dedicated from CM_COMPUTER_SYSTEM order by name";
  }

# --- execute query
$sth = $dbh->prepare($sql);
$sth->execute or die "step 2.01: SQL Error: $DBI::errstr\n";

# --- loop in output and add regex matches
while (@row = $sth->fetchrow_array) 
 {
# - decode line
  ($db_csid,$db_name, $db_cs, $db_type, $db_origin, $db_dedicated) = @row;

# - set db_name to upper case so that all compares are in upper case
$db_name=lc($db_name);
# - filter devices, origins and dedicated using regex 
   if ($useregex eq 'y')    
   {
# - commented - too many prints
#          if ($debugg eq 'y')
#             {
#               print "step 2.02: Evaluate server: $db_name cs_id: $db_csid\n"; 
#             }
# Check if device name is contained in the server list (case insensitive)
	  if ($db_name =~ /$servlist/i)  
		{
		 if (($filter_origin ne '' && $db_origin =~ /$filter_origin/i) || ($filter_origin eq ''))
			{ 
			 if (($filter_dedicated ne '' && $db_dedicated =~ /$filter_dedicated/i) || ($filter_dedicated eq ''))
			   {
				print "step 2.03: select device: $db_name cs_id: $db_csid origin: $db_origin dedicated: $db_dedicated\n";    
				if ($cslist eq '')
				{
					$cslist="\"$db_cs\"";
					$sslist="$db_name";
					$csidlist="$db_csid";
				}
				else
				{
					$cslist="$cslist,\"$db_cs\"";
					$sslist="$sslist,$db_name";
					$csidlist="$csidlist,$db_csid";
				}   
				$count++;
			   }
			}
		}
	}
# - filter devices, origins and dedicated without regex
	if ($useregex eq 'n')     
	{
# - Convert the list of servers to an array so we can match on 1-o-1
		@server_array = split ( /\|/, $servlist ); 
#                if ($debugg eq 'y')
#                 {
#                   print "step 2.04: Evaluate server: $db_name cs_id: $db_csid\n"; 
#                 }
# - Check if device name is contained in the server list array 
		if ( grep { $_ eq $db_name } @server_array )   
		{
# - Convert the list of origins to an array so we can match on 1-o-1
		@origin_array = split ( /\|/, $filter_origin );  
		if (($filter_origin ne '' && ( grep { $_ eq $db_origin } @origin_array ) ) || ($filter_origin eq ''))
			{ 
# - Convert the list of dedicated to an array so we can match on 1-o-1
			@dedicated_array = split ( /\|/, $filter_dedicated );  
			if (($filter_dedicated ne '' && ( grep { $_ eq $db_dedicated } @dedicated_array ) ) || ($filter_dedicated eq ''))
			  {
				print "step 2.05: select device: $db_name cs_id: $db_csid origin: $db_origin dedic: $db_dedicated\n";    
				if ($cslist eq '')
				{
					$cslist="\"$db_cs\"";
					$sslist="$db_name";
					$csidlist="$db_csid";					
				}
				else
				{
					$cslist="$cslist,\"$db_cs\"";
					$sslist="$sslist,$db_name";
					$csidlist="$csidlist,$db_csid";
				}   
				$count++;
			  }
                        else
                          {
				print "step 2.06: bypass device: $db_name cs_id: $db_csid origin: $db_origin dedic: $db_dedicated\n";   
                          } 
			}
                       else
                         {
         			print "step 2.07: bypass device: $db_name cs_id: $db_csid origin: $db_origin filter_origin: $filter_origin dedic: $db_dedicated\n";   
                         } 
		}

	}
}
print "step 2.08: We found $count matching devices in cm_computer_system (matching: $sslist) (csids: $csidlist)\n";

# --- when 0 matches are found we will stop here?
if ($stop_nomatch eq 'y' && $count eq '0')
 {
   if ($filter_origin eq '')
    {
        if ($probe eq 'y')
         {
            print "step 2.09: No server match was found in the cm_computer_system table\n";
         }
        else
         {  
           print (color("black on_white "),"step 2.09: No server match was found in the cm_computer_system table",color("reset"),"\n");
         }
       exit 1;
    }
   else
    {
        if ($probe eq 'y')
         {
            print "step 2.11: No server/origin match was found in the cm_computer_system table\n";
         }
        else
         {
           print (color("black on_white "),"step 2.11: No server/origin match was found in the cm_computer_system table",color("reset"),"\n");
         }
       exit 1;
    }
 } 

#
# Hub list for step 6
#

if ($delete_im eq 'y') 
  {
      $repcc=0;
# --- create uim robot list (via callback) (=old method that was not so performant for 1 device)
      if ($optioni eq 'y')
        {
          &uim_robot_list;
        } 
# --- create list of robots with corresponding hub address (via sql query, but csid must exist)
      else
        {    
           &uim_robot_hub;
        }
  }

#
# --- step3 - execute call back to discovery_server
#

if ($simulate eq 'y' && $debugg eq 'p')
{
   $target_addr="\/$UIM_Domain\/$UIM_Hub\/$UIM_Robot\/discovery_server";
   $cmd="remove_master_devices_by_cskeys";
#   print "=> pu -u $username -p ??? $target_addr $cmd $cslist \"\"\n";
   print (color("black on_green "),"=> pu -u $username -p ??? $target_addr $cmd $cslist \"\"",color("reset"),"\n");
}

if ($simulate eq 'n' || $debugg eq 'p')
{
  $sid = nimLogin($username, $password);
  if (!defined($sid))
    {
        if ($probe eq 'y')
         {
            print("step 3.01: Could not login to nimbus with username $username!\n");
         }
        else
         { 
            print (color("black on_white "),"step 3.01: Could not login to nimbus with username $username!",color("reset"),"\n");
         }
      exit(1);
    }
}

if ($simulate eq 'n')
{

$target_addr="\/$UIM_Domain\/$UIM_Hub\/$UIM_Robot\/discovery_server";
# print "Target: $target_addr\n";

# - Execute command to target
  if ($cslist ne "") 
  {
   $pds_in = Nimbus::PDS->new();
   $cmd="remove_master_devices_by_cskeys";

   $pds_in->string("csKeys", $cslist);
# 14/12/18
   if ($prevent_rediscovery eq 'y')
    {
      $pds_in->string("prevent_rediscovery", "true");
    }
   if ($debugg eq 'y')
     {
#        print "=> pu -u $username -p ??? $target_addr $cmd $cslist \"\"\n";
        print (color("black on_green "),"=> pu -u $username -p ??? $target_addr $cmd $cslist \"\"",color("reset"),"\n");
     }   
   ($rc, $pds_data) = nimNamedRequest($target_addr, $cmd, $pds_in->data(), $timeout);
   if ($rc)
     {
       $err = nimError2Txt($rc);
        if ($probe eq 'y')
         {        
            print "step 3.02: rc=$rc err=$err from nimNamedRequest for $cmd addr: $target_addr\n";
         }  
        else
         { 
            print (color("black on_white "),"step 3.02: rc=$rc err=$err from nimNamedRequest for $cmd addr: $target_addr",color("reset"),"\n");
         } 
       exit(1);
     }
   $pds_out = Nimbus::PDS->new($pds_data);
#   $pds_out->dump();
   doPDS($pds_out);
 }
} 
else
{
 if ($probe eq 'y')
  {  
      print "step 3.03: delete devices bypassed due to simulation run\n";
  }
else
  {  
      print (color("black on_yellow "),"step 3.03: delete devices bypassed due to simulation run",color("reset"),"\n");
  }
}

#
# --- step4 - delete QoS data for devices ($servlist)
#

if ($delete_qos eq 'n')
 {
   print "step 4.01: option -q was set to \"n\" (no delete QoS)\n";
   goto step5;
 }

# - get QoS overview details

# - better sql query to include also source that is ip address (mssql and mysql same)
if ($sqltype eq 'ms')
  {
    $sql = "select qd.origin, qd.robot, qd.probe, qd.qos, cs.name, qd.target, qd.r_table, qd.h_table, qd.table_id,cs.ip from  CM_COMPUTER_SYSTEM cs with(nolock),CM_DEVICE d with(nolock),CM_CONFIGURATION_ITEM ci with(nolock),CM_CONFIGURATION_ITEM_METRIC cim with(nolock),S_QOS_DATA qd with(nolock) where d.cs_id = cs.cs_id and ci.dev_id = d.dev_id and ci.dev_id = d.dev_id and cim.ci_id = ci.ci_id and qd.ci_metric_id = cim.ci_metric_id";
  }
else
  {
    $sql = "select qd.origin, qd.robot, qd.probe, qd.qos, cs.name, qd.target, qd.r_table, qd.h_table, qd.table_id,cs.ip from  CM_COMPUTER_SYSTEM cs,CM_DEVICE d,CM_CONFIGURATION_ITEM ci,CM_CONFIGURATION_ITEM_METRIC cim ,S_QOS_DATA qd where d.cs_id = cs.cs_id and ci.dev_id = d.dev_id and ci.dev_id = d.dev_id and cim.ci_id = ci.ci_id and qd.ci_metric_id = cim.ci_metric_id";
  }
# - old query version 1.3
# $sql = "SELECT origin, robot, probe, qos, source, target, r_table, h_table, table_id FROM S_QOS_DATA";
$sth = $dbh->prepare($sql);
$sth->execute or die "step 4.02: SQL Error: $DBI::errstr\n";

while (@row = $sth->fetchrow_array) 
 {
# - decode line
  ($qorigin,$qrobot,$qprobe,$qqos,$qsource,$qtarget,$qr_table,$qh_table,$qtable_id,$qip) = @row;
  $domatch=0;

# - compare in upper case
  $qsource=lc($qsource); 
# - filter origin using regex 
  if ($useregex eq 'y')    
   {
     if ($qsource =~ /$servlist/i) 
	{
	  if (($filter_origin ne '' && $qorigin =~ /$filter_origin/i) || ($filter_origin eq ''))
	    {
	      $domatch=1;
            }
	 else
	    {
	      $domatch=2;
	    }
       }
   }
# - filter origin without regex 
  if ($useregex eq 'n')    
   {
# - Convert the list of servers to an array so we can match on 1-o-1
	@server_array = split ( /\|/, $servlist );  
	if ( grep { $_ eq $qsource } @server_array ) 
	  {
# - Convert the list of origins to an array so we can match on 1-o-1	
		@origin_array = split ( /\|/, $filter_origin );  
		if (($filter_origin ne '' && ( grep { $_ eq $qorigin } @origin_array ) ) || ($filter_origin eq ''))    
	          {
			$domatch=1;
		  }
		else
		  {
			$domatch=2;
		  }
          }
   }
	

# - if there is a match add it in the table
    if ($domatch eq '1')
     {
       if ($simulate ne 'n' && $listqos eq 'y')
        {
           print "step 4.03: $count9: $qorigin $qrobot $qprobe $qqos $qsource $qip $qtarget $qr_table $qh_table $qtable_id\n";
        } 
       
# - put it in a table
       $qos9[$count9]="$qorigin\$qrobot\$qprobe\$qqos\$qsource\$qtarget\$qr_table\$qh_table\$qtable_id";
       $count9++;
     }
    $sqlcount++;
 }

# how many QoS did we see ?
$count9=$count9-1;
# print " \n";
print "step 4.04: selected: $count9 QoS (QoS total: $sqlcount) (Use -l\"y\" to see QoS details)\n";

# Get QoS related to MCS
if ($delete_mcs eq 'y' )
  {
    $qos_mcs_count=1;
    if ($sqltype eq 'ms')
     {
       $sql = "select source,qos,origin,target from S_QOS_DATA with(nolock) where qos like '%QOS_MCS_%' or qos like '%QOS_SSR_%';";
     }
    else
     {
       $sql = "select source,qos,origin,target from S_QOS_DATA where qos like '%QOS_MCS_%' or qos like '%QOS_SSR_%';";
     } 
    $sth = $dbh->prepare($sql);
    $sth->execute or die "step 4: SQL Error: $DBI::errstr\n";
    while (@row = $sth->fetchrow_array) 
     {
# - decode line
      ($qsource,$qqos,$qorigin,$qtarget) = @row;

# - compare in upper case
      $qsource=lc($qsource);
      $domatch=0;
# - filter origin using regex 
      if ($useregex eq 'y')    
       {
	  if ($qsource =~ /$servlist/i) 
		{
		if (($filter_origin ne '' && $qorigin =~ /$filter_origin/i) || ($filter_origin eq ''))
		{
			$domatch=1;
		}
		else
		{
			$domatch=2;
		}
		}
	} 
      if ($useregex eq 'n')    # - filter origin without regex 
       {
	  @server_array = split ( /\|/, $servlist );  # - Convert the list of servers to an array so we can match on 1-o-1
	  if ( grep { $_ eq $qsource } @server_array ) 
		{
		  @origin_array = split ( /\|/, $filter_origin );  # - Convert the list of origins to an array so we can match on 1-o-1	
		  if (($filter_origin ne '' && ( grep { $_ eq $qorigin } @origin_array ) ) || ($filter_origin eq ''))    
		   {
			$domatch=1;
		   }
	 	  else
		   {
			$domatch=2;
		   }
		}
	}

# - if there is a match add it in the table
      if ($domatch eq '1')
       {
         if ($simulate ne 'n' && $listqos eq 'y')
          {
           print "step 4.05: $qos_mcs_count: $qorigin $qqos $qsource $qtarget\n";
          } 
       
# - put it in a table
         $mcs_array[$qos_mcs_count]="$qorigin\$qqos\$qsource\$qtarget";
         $qos_mcs_count++;
       }
 }

# how many QoS did we see ?
  $qos_mcs_count=$qos_mcs_count-1;
# print " \n";
  print "step 4.06: selected: $qos_mcs_count MCS QoS (Use -l\"y\" to see QoS details)\n";

}

  if ($simulate eq 'y')
   {
     print "step 4.07: delete QoS bypassed due to simulation run\n";
     if ($delete_mcs eq 'y')
       { 
         print "step 4.08: delete MCS bypassed due to simulation run\n";
       }
     goto step5;
   }

print "step 4.09: Start deleting the QoS matches\n";

# - we have all info, process the table and print all info with extra query to get QoS details
$count9=1; 
$wait_count=0; 
while ($count9 < @qos9)
  {
     @numb9 = split (/\/, $qos9[$count9]); # create all keywords
     $yorigin=$numb9[0];
     $yrobot=$numb9[1];
     $yprobe=$numb9[2];
     $yqos = "$numb9[3]";
     $ysource = "$numb9[4]";
     $ytarget = "$numb9[5]";
     $yr_table = "$numb9[6]";
     $yh_table = "$numb9[7]";
     $ytable_id = "$numb9[8]";

# --- b&d tables are not referenced in s_qos_data, so create the table names ourself
     $yd_table = "DN_QOS_DATA_" . substr($yr_table, -4);
     $yb_table = "BN_QOS_DATA_" . substr($yr_table, -4);


# - delete selected qos entries
#       DELETE from 'RN_QOS_DATA_0014' where table_id='5429'
#       DELETE from 'HN_QOS_DATA_0014' where table_id='5429'
#       DELETE from S_QOS_DATA where table_id ='5429'
# - rn_qos_data (raw data)
     $sql = "DELETE from $yr_table where table_id='$ytable_id'";
     if ($listqos eq 'y')
      { 
        print "step 4.10: $count9: $sql\n";
      } 
     $sth = $dbh->prepare($sql);
     $num1=$sth->execute or die "step 4.11: SQL QoS $yqos detail Error: $DBI::errstr\n";
     if ($num1 eq '0E0') { $num1="0";}
     if ($verbose_sql eq 'y') {print "step 4.12: Deleted: $num1 QoS for: $yqos - $yr_table\n"; }
# - hn_qos_data (hourly history)
     $sql = "DELETE from $yh_table where table_id='$ytable_id'";
     if ($listqos eq 'y')
      { 
        print "step 4.13: $count9: $sql\n";
      } 
     $sth = $dbh->prepare($sql);
     $num2=$sth->execute or die "step 4.14: SQL QoS $yqos detail Error: $DBI::errstr\n"; 
     if ($num2 eq '0E0') { $num2="0";}
     if ($verbose_sql eq 'y') {print "step 4.15: Deleted: $num2 QoS for: $yqos - $yh_table\n"; }
# - bn_qos_data (baseline)
     $sql = "DELETE from $yb_table where table_id='$ytable_id'";
     if ($listqos eq 'y')
      { 
        print "step 4.16: $count9: $sql\n";
      } 
     $sth = $dbh->prepare($sql);
     $num3=$sth->execute or die "step 4.17: SQL QoS $yqos detail Error: $DBI::errstr\n"; 
     if ($num3 eq '0E0') { $num3="0";}
     if ($verbose_sql eq 'y') {print "step 4.18: Deleted: $num3 QoS for: $yqos - $yb_table\n"; }
# - dn_qos_data (daily history)
     $sql = "DELETE from $yd_table where table_id='$ytable_id'";
     if ($listqos eq 'y')
      { 
        print "step 4.19: $count9: $sql\n";
      } 
     $sth = $dbh->prepare($sql);
     $num4=$sth->execute or die "step 4.20: SQL QoS $yqos detail Error: $DBI::errstr\n"; 
     if ($num4 eq '0E0') { $num4="0";}
     if ($verbose_sql eq 'y') {print "step 4.21: Deleted: $num4 QoS for: $yqos - $yd_table\n"; }
# - s_qos_data (base entry)
     $sql = "DELETE from S_QOS_DATA where table_id='$ytable_id'";
     if ($listqos eq 'y')
      { 
        print "step 4.22: $count9: $sql\n";
      } 
     $sth = $dbh->prepare($sql);
     $num5=$sth->execute or die "step 4.23: SQL QoS $yqos detail Error: $DBI::errstr\n"; 
     if ($num5 eq '0E0') { $num5="0";}
     if ($verbose_sql eq 'y') {print "step 4.24: Deleted: $num5 QoS for: $yqos - S_QOS_DATA\n"; }
     $count9++;
  }
  $count9=$count9-1;
  print "step 4.25: Deleted $count9 QoS entries\n";
  
# - deletion of MCS QoS entries
if ($delete_mcs eq 'y')
  {
    $count9=1; 
    $wait_count=0; 
    while ($count9 < @mcs_array)
     {
       @numb9 = split (/\/, $mcs_array[$count9]); # create all keywords
       $yorigin=$numb9[0];
       $yqos = "$numb9[1]";
       $ysource = "$numb9[2]";
	 
# - s_qos_data MCS related
       $sql = "DELETE from S_QOS_DATA WITH (ROWLOCK) where qos like 'QOS_SSR_%' and source='$ysource'";
       if ($listqos eq 'y')
        { 
          print "step 4.26: $count9: $sql\n";
        } 
       $sth = $dbh->prepare($sql);
       $mcs1=$sth->execute or die "step 4.27: SQL QoS detail Error: $DBI::errstr\n"; 
       if ($mcs1 eq '0E0') { $mcs1="0";}
       if ($verbose eq 'y') {print "step 4.28: Deleted: $mcs1 QoS for: $ysource - QOS_SSR_\n"; }
# - s_qos_data MCS related
       $sql = "DELETE from S_QOS_DATA WITH (ROWLOCK) where qos like 'QOS_MCS_%' and source='$ysource'";
       if ($listqos eq 'y')
        { 
          print "step 4.29: $count9: $sql\n";
        } 
       $sth = $dbh->prepare($sql);
       $mcs2=$sth->execute or die "step 4.30: SQL QoS detail Error: $DBI::errstr\n";
       if ($mcs2 eq '0E0') { $mcs2="0";}
       if ($verbose eq 'y') {print "step 4.31: Deleted: $mcs2 QoS for: $ysource - QOS_MCS_\n"; }
       $count9++;
     }
   $count9=$count9-1;
   print "step 4.32: Deleted $count9 MCS QoS entries\n";

# - Section to delete MCS info
   @csid_array = split ( /,/, $csidlist ); 
   if (@csid_array) 
    {
	print "step 4.33: Deleting MCS entries (if any) for following cs_id: $csidlist\n";
        foreach my $csid (@csid_array) 
          {
# - SSRV2Profile #1
        	$sql = "UPDATE SSRV2Profile set status = 'delete' where cs_id='$csid'";
		$sth = $dbh->prepare($sql);
		$mcs3=$sth->execute or die "step 4.34: SQL QoS detail Error: $DBI::errstr\n"; 
		if ($mcs3 eq '0E0') { $mcs3="0";}
		if ($verbose eq 'y' && $mcs3 > 0) { print "step 4.35: Updated: $mcs3 csid: $csid - SSRV2Profile\n"; }
# - SSRV2Profile #2
		$sql = "DELETE from SSRV2Profile WITH (ROWLOCK) where poller = '$csid' and cs_id='$csid'";
		# - $sql = "DELETE from SSRV2Profile WITH (ROWLOCK) where cs_id='$csid'";
		$sth = $dbh->prepare($sql);
		$mcs6=$sth->execute or die "step 4.36: SQL QoS detail Error: $DBI::errstr\n"; 
		if ($mcs6 eq '0E0') { $mcs6="0";}
		if ($verbose eq 'y' && $mcs6 > 0) {print "step 4.37: Deleted: $mcs6 csid: $csid - SSRV2Profile\n"; }
# - SSRV2Poller
		$sql = "DELETE from SSRV2Poller WITH (ROWLOCK) where cs_id='$csid'";
		$sth = $dbh->prepare($sql);
		$mcs4=$sth->execute or die "step 4.38: SQL QoS detail Error: $DBI::errstr\n"; 
		if ($mcs4 eq '0E0') { $mcs4="0";}
		if ($verbose eq 'y' && $mcs4 > 0) {print "step 4.39: Deleted: $mcs4 csid: $csid - SSRV2Poller\n"; }	
# - SSRV2Device	
		$sql = "UPDATE SSRV2Device WITH (ROWLOCK) set status = 'delete' where cs_id='$csid'";
		# $sql = "DELETE from SSRV2Device WITH (ROWLOCK) where cs_id='$csid'";
		$sth = $dbh->prepare($sql);
		$mcs5=$sth->execute or die "step 4.40: SQL QoS detail Error: $DBI::errstr\n"; 
		if ($mcs5 eq '0E0') { $mcs5="0";}
		if ($verbose eq 'y' && $mcs5 > 0) {print "step 4.41: Deleted: $mcs5 csid: $csid - SSRV2Device\n"; }
          }
    } 
  else 
    {
	print "step 4.42: no MCS data deleted from SSRV2 tables\n";
    }
}

#
# New loop to wait for remote profiles to be deleted before continuing
#	
@csid_array1 = split ( /,/, $csidlist ); 
if (@csid_array1) 
{
  print "step 4.43: Waiting for remote profiles to be deleted for cs_id: $csidlist\n";
  foreach my $csid (@csid_array1) 
  {
	#$profile_count=1;
	#$total_sleep=60; # NOT seconds --- 15 mins - sleeping for 15 seconds = 4 increments per minute (4 x 15 minutes = 60)
	#$current_sleep=0;
     while ($profile_count > 0) 
     {
       	$sql = "select count(*) from SSRV2Profile where cs_id='$csid'";
	$sth = $dbh->prepare($sql);
	$mcs4=$sth->execute or die "step 4.44: SQL QoS detail Error: $DBI::errstr\n"; 
	if ($mcs4 eq '0E0') { $mcs4="0";}
	if ($verbose eq 'y' && $mcs4 > 0) { print "step 4.45: Updated: $mcs3 csid: $csid - SSRV2Profile\n"; }
	while (@row = $sth->fetchrow_array) 
        {

        	($profile_count) = @row;
	} 
	if ($profile_count > 0 && $current_sleep >= $total_sleep) 
          { # check the number of profiles for this device and loop until 0 or 15 mins
		sleep(15);
		$current_sleep++;
	  } 
        else 
          {
		break; # Giving up on waiting for these profiles to be deleted 
          }
     } # end of while
   } #end the foreach
}

#
# --- step5 - delete alarms in nas_transaction_log and nas_transaction_summary
#

step5:

# we have comma separated sslist with devices selected

if ($delete_alarm eq 'y')
  {
#     @rep = split(// , $sslist);
     @rep = split(/,/ , $sslist);
    $repcount=@rep;
    if ($simulate eq 'n')
     {
       $repcc=0;
# --- create list of divices
       while ($rep[$repcc] ne '')
         {     
           $zsource=$rep[$repcc];    
           if ($zsource ne $zsource_old)
             { 
# - nas_alarm_transaction_log
               print "step 5.01: Start deleting from nas_transaction_log for device: $zsource\n";
               $sql = "DELETE from nas_transaction_log where hostname = '$zsource'";
               $sth = $dbh->prepare($sql);
               $numb1=$sth->execute or die "step 5.02: SQL alarm detail Error: $DBI::errstr\n"; 
               if ($numb1 eq '0E0') { $numb1="0";}
               if ($verbose eq 'y') {print "step 5.03: Deleted: $numb1 alarms in nas_transaction log\n";}
               $sth->finish;
# - nas_alarm_transaction_summary
               print "step 5.04: Start deleting from nas_transaction_summary for device: $zsource\n";
               $sql = "DELETE from nas_transaction_summary where hostname = '$zsource'";
               $sth = $dbh->prepare($sql);
               $numb2=$sth->execute or die "step 5.05: SQL alarm detail Error: $DBI::errstr\n"; 
               if ($numb2 eq '0E0') { $numb2="0";}
               if ($verbose eq 'y') {print "step 5.06: Deleted: $numb2 alarms in nas_transaction_summary\n";}
# - free processing of this query
               $sth->finish;
             }
           $zsource_old=$zsource;
           $repcc++;
         }
     }
    else
     {
       print "step 5.07: delete alarms for $repcount devices (cs_id's) bypassed due to simulation run\n";
     } 
  }
else
  {
    print "step 5.08: delete alarms bypassed due to -a\"n\"\n";
  }

step6:
#
# --- step6 - delete device in hub/InfrastructureManager via callback: removerobot of the owning hub?
#

# - moved creation of hub list to beginning

if ($delete_im eq 'y')
  {
# --- obtain now the domain/hub name of this server
     @rep = split(/,/ , $sslist);
    $repcount=@rep;
#    if ($simulate eq 'n' || $debugg eq 'p')
#     {
       $repcc=0;
# --- create list of devices
       while ($rep[$repcc] ne '')
         {     
           $xsource=$rep[$repcc];    
           if ($xsource ne $xsource_old)
             { 
               $case_robot=$tab4{$xsource};
               $hubl=$tab2{$case_robot};
#               $hubl=$tab2{$xsource};
               $huba=$tab3{$hubl};
               if ($verbose eq 'y') {print "step 6.01: Search for device: $xsource (case: $case_robot) in hub: $hubl\n"}
               if ($hubl ne '')
                {
                  print "step 6.02: Delete robot: $xsource (case: $case_robot) from hub: $hubl\n";
# - create callback
                  $pds_in = Nimbus::PDS->new();
                  $cmd="removerobot";
#                  $pds_in->string("name", $xsource);
                  $pds_in->string("name", $case_robot);
                  if ($debugg ne 'p' && $simulate eq 'n')
                   {
                     $huba="$huba\/hub";
                     ($rc, $pds_data) = nimNamedRequest($huba, $cmd, $pds_in->data(), $timeout);
                     if ($rc)
                        {
                          $err = nimError2Txt($rc);
                          if ($probe eq 'y')
                            {
                              print "step 6.03: rc=$rc err=$err from nimNamedRequest for cmd: $cmd hub: $huba device: $xsource ($case_robot)\n";
                            }
                          else
                            {
                              print (color("black on_white "),"step 6.03: rc=$rc err=$err from nimNamedRequest for cmd: $cmd hub: $huba device: $xsource ($case_robot)",color("reset"),"\n");
                            }
                        }
                     print "step 6.04: Delete finished for robot: $xsource (case: $case_robot) from hub: $hubl\n";
                   }
                  if ($simulate eq 'y' && $debugg eq 'p')
                   {
                     $target_addr="$huba";
                     $cmd="removerobot";
#                     print "=> pu -u $username -p ??? $target_addr/hub $cmd $xsource \"\"\n";
#                     print "=> pu -u $username -p ??? $target_addr/hub $cmd $case_robot \"\"\n";
                     print (color("black on_green "),"=> pu -u $username -p ??? $target_addr/hub $cmd $case_robot \"\"",color("reset"),"\n");
                   }
                }
              else
                {
                  print "step 6.05: Delete hub registered robot: $xsource bypassed because no hub found\n";
                }

             }
           $xsource_old=$xsource;
           $repcc++;
         }
#     }
#    else
#     {
#       print "step 6.06: Delete in hub/IM for $repcount devices (cs_id's) bypassed due to simulation run\n";
#      } 
  }

step7:
#
# --- step7 - delete device in nas address table via callback: nameservice_delete
#
if ($delete_nas eq 'y')
  {
      $repcc=0;

# --- loop is list of servers
     @rep = split(/,/ , $sslist);
     $repcount=@rep;
     if ($simulate eq 'n')
      {
       $repcc=0;
# --- create list of devices
       while ($rep[$repcc] ne '')
         {     
           $ysource=$rep[$repcc];    
           if ($ysource ne $ysource_old)
             { 
#               if ($verbose eq 'y') {print "step 7.01: Search for device: $ysource\n"}
                  print "step 7.01: delete device: $ysource from nas address table\n";
# - added sleep 5 seconds to avoid communication error with probe
                  sleep(5);
# - create callback
                  $pds_in = Nimbus::PDS->new();
                  $cmd="nameservice_delete";
                  $pds_in->string("ip", $ysource);
                  ($rc, $pds_data) = nimNamedRequest($nasa, $cmd, $pds_in->data(), $timeout);
                  if ($rc)
                     {
                       $err = nimError2Txt($rc);
                       if ($probe eq 'y')
                         {
                            print("step 7.02: rc=$rc err=$err from nimNamedRequest for $cmd\n");
                         }
                       else 
                         {
                            print (color("black on_white "),"step 7.02: rc=$rc err=$err from nimNamedRequest for $cmd",color("reset"),"\n");
                         }
                     }

             }
           $ysource_old=$ysource;
           $repcc++;
         }
     }
    else
     {
       print "step 7.03: Delete from nas address table for $repcount devices bypassed due to simulation run\n";
     } 
  }
else
  {
#    print "step 7.04: Delete device in nas bypassed due to -n\"n\"\n";
  }

# --- disconnect db
$dbh->disconnect();

print "step 8.00: normal end of $pgm\n";
exit 0;

#
# --- sub procedures ---
#

sub uim_robot_list
{
#
# --- Block to read Nimsoft Robots and put them in a table
#

$ihub=0;
$robot_count=0;

# --- Login Nimsoft
$sid = nimLogin($username, $password);
if (!defined($sid))
  {
    print("Could not login to nimbus with username: $username\n");
    exit(1);
  }

# --- Get Hub address
($rc, $hub_addr) = nimGetVarStr(NIMV_HUBADDR);
if ($rc)
  {
    if ($debugg == 2) { print "$prgname rc=$rc from nimGetVarStr NIMV_HUBADDR\n";}
    exit(2);
  }

# --- gethubs
$pds_in = Nimbus::PDS->new();
($rc, $pds_data) = doNimNamedRequest($hub_addr, 'gethubs', $pds_in, $request_timeout, $request_retries);
if ($rc)
  {
     print "rc=$rc of nimNamedRequest 'gethubs' from hub $hub_addr\n";
     exit(3);
  }
$pds_out = Nimbus::PDS->new($pds_data);

# --- Loop in hub list
$ihub = 0;
while(1)
 {
     $pds = $pds_out->getTable('hublist', PDS_PDS, $ihub);
     last if (! defined $pds);
     $domain_name = $pds->get('domain', PDS_PCH);
     $hub_name = $pds->get('name', PDS_PCH);
     if ($domain_name eq $UIM_Domain)
       {
         $hub_addr = $pds->get('addr', PDS_PCH);
         $hub_status = $pds->get('status', PDS_PCH);
         $hub_robot = $pds->get('robotname', PDS_PCH);
# - create hub table
         $tab3{$hub_name}="$hub_addr";
         if ($verbose eq 'y') {print "Processing hub: $hub_name status: $hub_status\n";}

# --- Loop in the robots, but only if hub_status = 0
         if ($hub_status eq '0')
           {
             $rch=doRobots($domain_name, $hub_addr, $hub_name); 
           }
         else
          {
#            print "Hub: $hub_name bypasses not active\n";
          } 
         $ihub++;
      }
    else
      {
#        print "Bypass domain: $domain_name hub: $hub_name\n";
        $ihub++;
      } 
 }

#  if ($verbose eq 'y') {print "step 6: Created robot list based on $ihub hubs and $robot_count robots from online Nimsoft\n";}

 

#
# --- End of Nimsoft Robot List
#
}


sub doNimNamedRequest
{
    my $address = shift;
    my $command = shift;
    my $data = shift;
    my $timeout = shift;
    my $retries = shift;

    ($rc, $pds_data);
    for (my $itry = 0; $itry < $retries; $itry++)
    {
#        print "nimNamedRequest addr: $address cmd: $command data: $data timeout: $timeout \n";
        ($rc, $pds_data) = nimNamedRequest($address, $command, $data, $timeout);
        last if (! $rc);
        if ($debugg == 2) 
         {
           if ($command ne '_status' && $command ne 'list_subscribers' && $command ne 'getrobots' && $command ne 'probe_list')
            {
              print "rc=$rc for nimNamedRequest address=$address command=$command timeout=$timeout try=$itry\n";
            }
         }
    }
    return($rc, $pds_data);
}

sub doPDS
{
  my $pds = shift;

  $level++;
  $pds->rewind();
  while(1)
  {
    ($rc, $key, $type, $size, $data) = pdsGetNext($pds->data());
    last if ($rc != NIME_OK);
    $string = $pds_type2string{$type};
#    print("level=$level rc=$rc key=$key type=$string size=$size data=$data\n");
#    print("$level $key $string $size $data\n");
    print("$key: $string $size $data\n");
    if ($string eq "PDS_PPDS" || $string eq "PDS_PPCH")
    {
      $pds_data = $pds->get($key, $pds_type{"PDS_PDS"});
      doPDS($pds_data);
    }
    elsif ($string eq "PDS_PDS" )
    {
      $pds_data = Nimbus::PDS->new($data);
      doPDS($pds_data);
    }
  }
  $level--;
}

sub doRobots
{
    my $domain_name = shift;
    my $hub_addr = shift;
    my $hub_name = shift;
    $robot_status=2;
    $hub_robot_ok=0;
    $hub_robot_nok=0;

#    print "$prgname (doRobots) - domain_name=$domain_name hub_addr=$hub_addr hub_name=$hub_name\n";
    my $pds_in = Nimbus::PDS->new();
    my $pds_data;
    my ($rc, $pds_data) = doNimNamedRequest($hub_addr, 'getrobots', $pds_in, $request_timeout, $request_retries);
    if ($rc)
    {
        print "  rc=$rc from nimNamedRequest 'getrobots' for hub $hub_name ($hub_addr)\n";
        return $rc;
    }
    else
    {
      $hbstat=0;
    }

    my $pds_out = Nimbus::PDS->new($pds_data);
    $irobot = 0;
    while(1)
    {
        my $pds = $pds_out->getTable('robotlist', PDS_PDS, $irobot);
        last if (! defined $pds);

        $robot_addr = $pds->get('addr', PDS_PCH);
        $robot_name = $pds->get('name', PDS_PCH);
        $robot_status = $pds->get('status', PDS_INT);

        $irobot++;
        $robot_count++;
#        print "Add robot $robot_count: $robot_name in install_table for hub: $hub_name \n";
#         push(@tab1, "$domain_name\$hub_name\$robot_name");
#         $tab2{$robot_name}="$domain_name\/$hub_name\/$robot_name";
# we only need the hub name
         if ($verbose eq 'y') {print "Add robot $robot_count: $robot_name in tab2_table for hub: $hub_name \n";}
         $tab2{$robot_name}="$hub_name";
   }

  return $hbstat;
}


sub uim_robot_hub
{
# --- prepare sql query (nimbus_type: 0= 1=robot 2=hub)
  if ($sqltype eq 'ms')
   {
     $sql = "SELECT ip,os_major,os_minor,domain,hub,robot,origin,is_hub,robot_active,user_tag_1,user_tag_2,address,dev_name FROM CM_NIMBUS_ROBOT r with(nolock),CM_DEVICE d with(nolock) where r.dev_id = d.dev_id";
   }
  else
   {
     $sql = "SELECT ip,os_major,os_minor,domain,hub,robot,origin,is_hub,robot_active,user_tag_1,user_tag_2,address,dev_name FROM CM_NIMBUS_ROBOT r,CM_DEVICE d where r.dev_id = d.dev_id";
   } 

# --- execute query
  $sth = $dbh->prepare($sql);
  $sth->execute or die "step 6 - SQL Error create robot list: $DBI::errstr\n";

# --- loop in output
  while (@row = $sth->fetchrow_array) 
   {
# - decode line
     ($o_ip,$o_os_major,$o_os_minor,$o_domain,$o_hub,$o_robot,$o_origin,$o_is_hub,$o_robot_active,$o_user_tag1,$o_user_tag2,$o_address,$o_dev_name) = @row;
     $count++;
     $o_robot_lc=lc($o_robot);
# - create hub table (only if device is a hub)
     if ($o_is_hub eq '1')
      {
         $tab3{$o_hub}="$o_address";
#         print "added hub: $o_hub addr: $o_address\n";
      }
# - create robot table
     $tab2{$o_robot}="$o_hub";
#     print "added robot: $o_robot hub: $o_hub\n";

# 07/05/19 - added to find robot with the correct case that is needed by the removerobot command
# - create robot lc case table 
     $tab4{$o_robot_lc}="$o_robot";
#     print "added robot: $o_robot hub: $o_hub\n";
   }
}

# --- $file_input is set to the path and file name
sub read_input
{
$serv_count=0;
# --- open server input file
open (GLOBALS, "$file_input") or die "step 1.01: Error: Can't open $file_input for read: $!";
@lines_server = <GLOBALS>;
close GLOBALS or die "step 1.02: Error: Cannot close $file_input: $!"; 

# --- loop in servernames
foreach my $lineserver (@lines_server)
  {
    $reqcount++;
# --- remove blanks
    $lineserver =~ s/^\s+|\s+$//g;
    $lineserver=lc($lineserver);
    if ($display_server eq '')
     {
       $display_server=$lineserver;
     }
    else
     {
       if ($lineserver ne '')
        {
          $display_server="$display_server|$lineserver";
        }
     }
    $serv_count++; 
  }
print "step 1: We found: $serv_count servers in input file: $file_input\n";
}

sub getparams
{
  usage() unless @ARGV gt 0;
  my %Opts;
  getopts('h:d:s:l:y:q:o:c:k:a:i:v:r:x:m:n:f:w:p:t:',\%Opts);
  $file_list=$Opts{h};
  $file_input=$Opts{f};
  $simulate=$Opts{s};
  $debugg=$Opts{d};
  $listqos=$Opts{l};
  $sqlopt=$Opts{y};
  $delete_qos=$Opts{q};
  $delete_alarm=$Opts{a};
  $delete_nas=$Opts{n};
  $delete_im=$Opts{i};
  $delete_mcs=$Opts{m};
  $filter_origin=$Opts{o}; 
  $filter_dedicated=$Opts{c}; 
  $kill=$Opts{k};
  $verbose=$Opts{v};
  $verbose_sql=$Opts{t};
  $useregex=$Opts{r};
  $stop_nomatch=$Opts{x};
  $probe=$Opts{w};
  $prevent_rediscovery=$Opts{p};

  if ($filter_dedicated ne '')
   {
     $delete_qos="n";
   }
  if ($prevent_rediscovery eq '')
   {
     $prevent_rediscovery="n";
   }
  if ($debugg eq '')
   {  
     $debugg="n";
   } 
  if ($listqos eq '')
   {  
     $listqos="n";
   } 
  if ($stop_nomatch eq '')
   {  
     $stop_nomatch="y";
   }
  if ($kill eq 'y')
   {
     $simulate="n";
   }  
   if ($useregex eq '' || $useregex eq 'n')
   {  
     $useregex="n";
   }   
  if ($simulate eq '' || $simulate eq 'y')
   {  
     $simulate="y";
     print "*** Simulation run: no deletes are performed ***\n";  
     print "\n";
   }
  if ($file_list eq '' && $file_input eq '')
   {
      print "You MUST define the server/device to delete via the option -h (in regex format) or -f (file input)\n";
      exit 2;
   }
  if ($sqlopt eq '1' || $sqlopt eq '')
   {
      $sqltype="ms";
   }
  if ($sqlopt eq '2')
   {
      $sqltype="my";
   }
  if ($delete_qos eq '')
   {  
     $delete_qos="y";
   } 
  if ($delete_im eq '')
   {  
     $delete_im="y";
   } 
  if ($delete_nas eq '')
   {  
     $delete_nas="n";
   } 
  if ($delete_mcs eq '')
   {  
     $delete_mcs="y";
   } 
  if ($delete_alarm eq '')
   {  
     $delete_alarm="y";
   } 
  if ($verbose eq '')
   {  
     $verbose="y";
   } 
  if ($verbose_sql eq '')
   {  
     $verbose_sql="n";
   } 
  if ($probe eq '')
   {  
     $probe="n";
   } 

# --- get credentials and settings from common file
($Server_With_Rest_Interface, $RestServicePort, $username, $password,$UIM_Domain,$UIM_Hub,$UIM_Robot,$sql_server,$sql_user,$sql_password,$sql_db,$sql_type,$use_https) = nimsoft_generic::getRestData();


# -- de-crypt (need $encoded)
$decoded = decode_base64($password);
$password = RC4($crkey, $decoded);

$decoded = decode_base64($sql_password);
$sql_password = RC4($crkey, $decoded);

$nasa="/$UIM_Domain/$UIM_Hub/$UIM_Robot/nas";

  if ($nimuser eq '') 
   {
     $nimuser=$UIM_username;
   }
  if ($nimpass eq '')
   {
     $nimpass=$password;
   }

  if ($sql_type eq 'mssql')
   {
      $sqltype="ms";
   } 
  if ($sql_type eq 'mysql')
   {
      $sqltype="my";
   } 
  if ($sql_user eq 'trusted')
      {
         $trusted=yes;
      }

}

sub usage
 {
   print "      \n";
   print "Example: perl nimsoft_delete_device.pl -h\"server1|server2\"\n";
   print "(version: $pgm)\n";
   print "      \n";
   print (color("black on_green ")," Selection parameters:",color("reset"),"\n");
   print "  -h: device(s) to delete (| separated)\n";
   print "  -f: (opt) path/file to input file with 1 server/line to delete\n";
   print "  -o: (opt) filter on origin\n";
   print "  -c: (opt) filter on dedicated\n"; 
   print "      (Device, VirtualMachine, Host, HostSystem) \n";
   print "  -r: enable regex expressions on flags -h,-o,-c (y,n) default: n\n";
   print (color("black on_green ")," Delete parameters:",color("reset"),"\n");
   print "  -p: prevent rediscovery (y,n) Default: n\n";
   print "  -q: delete QoS (y,n) default: y\n";
   print "      (QoS will not be deleted if -c is used)\n";
   print "  -i: delete in hub/IM (y,n) default: y\n";
   print "  -a: delete Alarm history (y,n) default: y\n";
   print "  -m: delete MCS profile records and QoS entries (y,n) default: y\n";
   print "  -n: delete in nas addresstable (y,n) default: n\n";
   print (color("black on_green ")," Display parameters:",color("reset"),"\n");
   print "  -l: list detailed list of selected QoS to delete (y,n) default: n\n";
   print "  -v: verbose (y,n) default: y \n";
   print "  -t: verbose on sql QoS delete (y,n) default: n \n";
   print (color("black on_green ")," Execution parameters:",color("reset"),"\n");
   print "  -s: simulate run (y,n) default: y\n";
#   print "  -x: stop processing when no match is found in cm_computer_system, default: y\n";
   print "      \n";
   exit;
 }